@@ -25,8 +25,7 @@
 
 #include "common.h"
 #include "backendgvfs.h"
-#include "passworddialog.h"
-#include "main.h"
+#include "mountoperation.h"
 
 typedef struct _GigoloBackendGVFSPrivate			GigoloBackendGVFSPrivate;
 
@@ -52,6 +51,7 @@ typedef struct
 {
 	GigoloBackendGVFS *self;
 	GtkWidget *dialog;
+	gboolean show_errors;
 } MountInfo;
 
 struct _GigoloBackendGVFSPrivate
@@ -446,7 +446,10 @@ static void mount_ready_cb(GFile *location, GAsyncResult *res, MountInfo *mi)
 	if (error != NULL && ! g_error_matches(error, G_IO_ERROR, G_IO_ERROR_ALREADY_MOUNTED))
 	{
 		gchar *msg = g_strdup_printf(_("Connecting to \"%s\" failed."), uri);
-		g_signal_emit(mi->self, signals[OPERATION_FAILED], 0, msg, error->message);
+		if (mi->show_errors)
+			g_signal_emit(mi->self, signals[OPERATION_FAILED], 0, msg, error->message);
+		else
+			verbose("%s (%s)", msg, error->message);
 		g_free(msg);
 	}
 
@@ -461,48 +464,8 @@ static void mount_ready_cb(GFile *location, GAsyncResult *res, MountInfo *mi)
 }
 
 
-static void set_password_cb(GMountOperation *op, G_GNUC_UNUSED gchar *message, gchar *default_user,
-							gchar *default_domain, GAskPasswordFlags flags, const gchar *domain)
-{
-	GMountOperationResult result;
-	GtkWidget *dialog;
-
-	dialog = gigolo_password_dialog_new(flags, default_user,
-		(domain != NULL) ? domain : default_domain);
-
-	if (gtk_dialog_run(GTK_DIALOG(dialog)) == GTK_RESPONSE_OK)
-	{
-		result = G_MOUNT_OPERATION_HANDLED;
-
-		if (flags & G_ASK_PASSWORD_NEED_DOMAIN)
-			g_mount_operation_set_domain(op,
-				gigolo_password_dialog_get_domain(GIGOLO_PASSWORD_DIALOG(dialog)));
-		if (flags & G_ASK_PASSWORD_NEED_USERNAME)
-			g_mount_operation_set_username(op,
-				gigolo_password_dialog_get_username(GIGOLO_PASSWORD_DIALOG(dialog)));
-		if (flags & G_ASK_PASSWORD_NEED_PASSWORD)
-		{
-			g_mount_operation_set_password(op,
-				gigolo_password_dialog_get_password(GIGOLO_PASSWORD_DIALOG(dialog)));
-			/* TODO make this configurable? */
-			/* g_mount_operation_set_password_save(op, G_PASSWORD_SAVE_FOR_SESGIGOLO); */
-			/* g_mount_operation_set_password_save(op, G_PASSWORD_SAVE_NEVER); */
-			g_mount_operation_set_password_save(op, G_PASSWORD_SAVE_PERMANENTLY);
-		}
-	}
-	else
-	{
-		result = G_MOUNT_OPERATION_ABORTED;
-	}
-
-	gtk_widget_destroy(dialog);
-
-	g_mount_operation_reply(op, result);
-}
-
-
 void gigolo_backend_gvfs_mount_uri(GigoloBackendGVFS *backend, const gchar *uri,
-								 const gchar *domain, GtkWidget *dialog)
+								   GtkWindow *parent, GtkWidget *dialog, gboolean show_errors)
 {
 	GMountOperation *op;
 	GFile *file;
@@ -511,13 +474,12 @@ void gigolo_backend_gvfs_mount_uri(GigoloBackendGVFS *backend, const gchar *uri,
 	g_return_if_fail(uri != NULL);
 	g_return_if_fail(backend != NULL);
 
-	op = g_mount_operation_new();
+	op = gigolo_mount_operation_new(GTK_WINDOW(parent));
 	file = g_file_new_for_uri(uri);
 	mi = g_new0(MountInfo, 1);
 	mi->self = backend;
 	mi->dialog = dialog;
-
-	g_signal_connect(op, "ask-password", G_CALLBACK(set_password_cb), (gchar*) domain);
+	mi->show_errors = show_errors;
 
 	g_file_mount_enclosing_volume(file, G_MOUNT_MOUNT_NONE, op, NULL,
 		(GAsyncReadyCallback) mount_ready_cb, mi);
@@ -534,3 +496,183 @@ gchar *gigolo_backend_gvfs_get_volume_identifier(GVolume *volume)
 }
 
 
+gchar **gigolo_backend_gvfs_get_smb_shares_from_uri(const gchar *uri)
+{
+	gchar **shares = NULL;
+	GList *l, *shares_list = NULL;
+	GFile *file;
+	GFileInfo *info;
+	GError *error = NULL;
+	GFileEnumerator *e;
+
+	g_return_val_if_fail(uri != NULL, NULL);
+
+	verbose("Querying \"%s\" for available shares", uri);
+
+	file = g_file_new_for_uri(uri);
+
+	e = g_file_enumerate_children(file,
+		G_FILE_ATTRIBUTE_MOUNTABLE_CAN_MOUNT "," G_FILE_ATTRIBUTE_STANDARD_NAME,
+		G_FILE_QUERY_INFO_NONE, NULL, &error);
+
+	if (error != NULL)
+	{
+		verbose("%s: %s", G_STRFUNC, error->message);
+		g_error_free(error);
+	}
+	else
+	{
+		guint i, len;
+
+		while ((info = g_file_enumerator_next_file(e, NULL, NULL)) != NULL)
+		{
+			shares_list = g_list_append(shares_list, g_strdup(g_file_info_get_name(info)));
+
+			g_object_unref(info);
+		}
+		g_object_unref(e);
+
+		len = g_list_length(shares_list);
+		if (len > 0)
+		{
+			i = 0;
+		shares = g_new(gchar*, len + 1);
+
+		for (l = shares_list; l != NULL; l = g_list_next(l))
+		{
+			shares[i] = l->data;
+			i++;
+		}
+		shares[i] = NULL;
+		}
+		g_list_free(shares_list);
+	}
+
+	g_object_unref(file);
+
+	return shares;
+}
+
+
+gchar **gigolo_backend_gvfs_get_smb_shares(const gchar *hostname, const gchar *user, const gchar *domain)
+{
+	gchar **shares = NULL;
+	gchar *uri;
+
+	g_return_val_if_fail(hostname != NULL, NULL);
+
+	uri = g_strdup_printf("smb://%s%s%s%s%s/",
+		(NZV(domain)) ? domain : "",
+		(NZV(domain)) ? ";" : "",
+		(NZV(user)) ? user : "",
+		(NZV(user) || NZV(domain)) ? "@" : "",
+		hostname);
+
+	shares = gigolo_backend_gvfs_get_smb_shares_from_uri(uri);
+	g_free(uri);
+
+	return shares;
+}
+
+
+GigoloHostUri **gigolo_backend_gvfs_browse_network(const gchar *uri)
+{
+	GigoloHostUri *h, **hosts = NULL;
+	GList *l, *hosts_list = NULL;
+	GFile *file;
+	GFileInfo *info;
+	GError *error = NULL;
+	GFileEnumerator *e;
+
+	/* If a uri is passed, we assume this is an URI pointing to a workgroup like
+	 * "smb://WORKGROUP/", if passed NULL, we use "smb://" to search for workgroups */
+	if (uri == NULL)
+		uri = "smb://";
+
+	verbose("Querying \"%s\" for available groups/hosts", uri);
+
+	file = g_file_new_for_uri(uri);
+
+	e = g_file_enumerate_children(file,
+		G_FILE_ATTRIBUTE_MOUNTABLE_CAN_MOUNT ","
+		G_FILE_ATTRIBUTE_STANDARD_TARGET_URI ","
+		G_FILE_ATTRIBUTE_STANDARD_DISPLAY_NAME ","
+		G_FILE_ATTRIBUTE_STANDARD_ICON,
+		G_FILE_QUERY_INFO_NONE, NULL, &error);
+
+	if (error != NULL)
+	{
+		verbose("%s: %s", G_STRFUNC, error->message);
+		g_error_free(error);
+	}
+	else
+	{
+		guint i, len;
+		const gchar *h_uri;
+
+		while ((info = g_file_enumerator_next_file(e, NULL, NULL)) != NULL)
+		{
+			h_uri = g_file_info_get_attribute_string(info, G_FILE_ATTRIBUTE_STANDARD_TARGET_URI);
+			if (h_uri != NULL && g_str_has_prefix(h_uri, "smb://"))
+			{
+				h = g_new(GigoloHostUri, 1);
+				h->name = g_strdup(g_file_info_get_display_name(info));
+				h->uri = g_strdup(h_uri);
+				h->icon = g_object_ref(g_file_info_get_icon(info));
+				hosts_list = g_list_append(hosts_list, h);
+			}
+			g_object_unref(info);
+		}
+		g_object_unref(e);
+
+		len = g_list_length(hosts_list);
+		if (len > 0)
+		{
+			i = 0;
+		hosts = g_new(GigoloHostUri*, len + 1);
+
+		for (l = hosts_list; l != NULL; l = g_list_next(l))
+		{
+			hosts[i] = (GigoloHostUri*) l->data;
+			i++;
+		}
+		hosts[i] = NULL;
+		}
+		g_list_free(hosts_list);
+	}
+	g_object_unref(file);
+
+	return hosts;
+}
+
+
+gpointer gigolo_backend_gvfs_get_share_icon(void)
+{
+	if (gtk_check_version(2, 14, 0) == NULL)
+	return g_themed_icon_new("folder-remote");
+	else
+		return NULL;
+}
+
+
+const gchar *const *gigolo_backend_gvfs_get_supported_uri_schemes(void)
+{
+	return g_vfs_get_supported_uri_schemes(g_vfs_get_default());
+}
+
+
+gboolean gigolo_backend_gvfs_is_scheme_supported(const gchar *scheme)
+{
+	const gchar *const *schemes = gigolo_backend_gvfs_get_supported_uri_schemes();
+	guint i;
+
+	g_return_val_if_fail(scheme != NULL, FALSE);
+
+	for (i = 0; schemes[i] != NULL; i++)
+	{
+		if (gigolo_str_equal(schemes[i], scheme))
+			return TRUE;
+	}
+
+	return FALSE;
+}
