@@ -28,17 +28,22 @@
 #include <string.h>
 #include <libintl.h>
 
-#include "main.h"
+#include "common.h"
 #include "settings.h"
+#include "bookmark.h"
 #include "window.h"
+#include "backendgvfs.h"
+#include "singleinstance.h"
 
 
 static gboolean show_version = FALSE;
-static gboolean verbose_mode = FALSE;
 static gboolean list_schemes = FALSE;
+static gboolean new_instance = FALSE;
+gboolean verbose_mode = FALSE;
 
 static GOptionEntry cli_options[] =
 {
+	{ "new-instance", 'i', 0, G_OPTION_ARG_NONE, &new_instance, N_("Ignore running instances, enforce opening a new instance"), NULL },
 	{ "list-schemes", 'l', 0, G_OPTION_ARG_NONE, &list_schemes, N_("Print a list of supported URI schemes"), NULL },
 	{ "verbose", 'v', 0, G_OPTION_ARG_NONE, &verbose_mode, N_("Be verbose"), NULL },
 	{ "version", 'V', 0, G_OPTION_ARG_NONE, &show_version, N_("Show version information"), NULL },
@@ -46,37 +51,12 @@ static GOptionEntry cli_options[] =
 };
 
 
-#ifdef DEBUG
-void debug(gchar const *format, ...)
-{
-	va_list args;
-	va_start(args, format);
-	g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_DEBUG, format, args);
-	va_end(args);
-}
-#endif
-
-
-void verbose(gchar const *format, ...)
-{
-#ifndef DEBUG
-	if (verbose_mode)
-#endif
-	{
-		va_list args;
-		va_start(args, format);
-		g_logv(G_LOG_DOMAIN, G_LOG_LEVEL_MESSAGE, format, args);
-		va_end(args);
-	}
-}
-
-
 static void print_supported_schemes(void)
 {
 	const gchar* const *supported;
 	gint j;
 
-	supported = g_vfs_get_supported_uri_schemes(g_vfs_get_default());
+	supported = gigolo_backend_gvfs_get_supported_uri_schemes();
 	for (j = 0; supported[j] != NULL; j++)
 	{
 		g_print("%s\n", supported[j]);
@@ -87,6 +67,7 @@ static void print_supported_schemes(void)
 gint main(gint argc, gchar** argv)
 {
 	GigoloSettings *settings;
+	GigoloSingleInstance *gis = NULL;
 	const gchar *vm_impl;
 	gchar *accel_filename;
 	GOptionContext *context;
@@ -122,6 +103,17 @@ gint main(gint argc, gchar** argv)
 		return EXIT_SUCCESS;
 	}
 
+	if (! new_instance)
+	{
+		gis = gigolo_single_instance_new();
+		if (gigolo_single_instance_is_running(gis))
+		{
+			gigolo_single_instance_present(gis);
+			g_object_unref(gis);
+			exit(0);
+		}
+	}
+
 	verbose("Gigolo %s (GTK+ %u.%u.%u, GLib %u.%u.%u)",
 		VERSION,
 		gtk_major_version, gtk_minor_version, gtk_micro_version,
@@ -138,16 +130,23 @@ gint main(gint argc, gchar** argv)
 		g_setenv("GIO_USE_VOLUME_MONITOR", vm_impl, 0);
 
 	window = gigolo_window_new(settings);
-    g_signal_connect(window, "destroy", G_CALLBACK(gtk_main_quit), NULL);
 
-	if (gigolo_settings_get_boolean(settings, "start-in-systray"))
+	if (gis != NULL)
+		gigolo_single_instance_set_parent(gis, GTK_WINDOW(window));
+
+	if (gigolo_settings_get_boolean(settings, "start-in-systray") &&
+		gigolo_settings_get_boolean(settings, "show-in-systray"))
+	{
 		gdk_notify_startup_complete();
+	}
 	else
 		gtk_widget_show(window);
 
 	gtk_main();
 
 	g_object_unref(settings);
+	if (gis != NULL)
+		g_object_unref(gis);
 
 	gtk_accel_map_save(accel_filename);
 	g_free(accel_filename);
