@@ -37,7 +37,7 @@
 #include "preferencesdialog.h"
 #include "backendgvfs.h"
 #include "mountdialog.h"
-#include "main.h"
+#include "browsenetworkpanel.h"
 
 
 typedef struct _GigoloWindowPrivate			GigoloWindowPrivate;
@@ -55,8 +55,10 @@ struct _GigoloWindowPrivate
 	GigoloBackendGVFS	*backend_gvfs;
 
 	GtkWidget		*vbox;
-	GtkWidget		*hbox;
+	GtkWidget		*hbox_pane;
+	GtkWidget		*hbox_view;
 
+	GtkWidget		*browse_panel;
 	GtkWidget		*treeview;
 	GtkWidget		*iconview;
 	GtkWidget		*swin_treeview;
@@ -77,7 +79,6 @@ struct _GigoloWindowPrivate
 	GtkWidget		*systray_icon_popup_menu;
 
 	guint			 autoconnect_timeout_id;
-	gboolean		 window_hidden;
 };
 
 enum
@@ -98,41 +99,6 @@ enum
 G_DEFINE_TYPE(GigoloWindow, gigolo_window, GTK_TYPE_WINDOW);
 
 
-static gboolean gigolo_window_state_event(GtkWidget *widget, GdkEventWindowState *event)
-{
-	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(widget);
-	gboolean show_systray_icon = gigolo_settings_get_boolean(priv->settings, "show-in-systray");
-
-	if (show_systray_icon)
-	{
-		if (event->changed_mask & GDK_WINDOW_STATE_ICONIFIED)
-		{
-			if (event->new_window_state & GDK_WINDOW_STATE_ICONIFIED)
-				priv->window_hidden = TRUE;
-			else
-				priv->window_hidden = FALSE;
-		}
-		if (event->changed_mask & GDK_WINDOW_STATE_WITHDRAWN)
-		{
-			if (event->new_window_state & GDK_WINDOW_STATE_WITHDRAWN)
-				priv->window_hidden = TRUE;
-			else
-				priv->window_hidden = FALSE;
-		}
-
-		if (priv->window_hidden && show_systray_icon)
-		{
-			gtk_window_set_skip_taskbar_hint(GTK_WINDOW(widget), TRUE);
-		}
-		else if (! priv->window_hidden)
-		{
-			gtk_window_set_skip_taskbar_hint(GTK_WINDOW(widget), FALSE);
-		}
-	}
-	return FALSE;
-}
-
-
 static void remove_autoconnect_timeout(GigoloWindow *window)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
@@ -145,19 +111,19 @@ static void remove_autoconnect_timeout(GigoloWindow *window)
 }
 
 
-static gboolean gigolo_window_delete_event(GtkWidget *widget, G_GNUC_UNUSED GdkEventAny *event)
+static void gigolo_window_destroy(GigoloWindow *window)
 {
-	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(widget);
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 	gint geo[5];
 
-	remove_autoconnect_timeout(GIGOLO_WINDOW(widget));
+	remove_autoconnect_timeout(window);
 
 	if (gigolo_settings_get_boolean(priv->settings, "save-geometry"))
 	{
-		gtk_window_get_position(GTK_WINDOW(widget), &geo[0], &geo[1]);
-		gtk_window_get_size(GTK_WINDOW(widget), &geo[2], &geo[3]);
-		if (priv->window_hidden && gdk_window_get_state(
-			gigolo_widget_get_window(widget)) & GDK_WINDOW_STATE_MAXIMIZED)
+		gtk_window_get_position(GTK_WINDOW(window), &geo[0], &geo[1]);
+		gtk_window_get_size(GTK_WINDOW(window), &geo[2], &geo[3]);
+		if (gdk_window_get_state(
+				gigolo_widget_get_window(GTK_WIDGET(window))) & GDK_WINDOW_STATE_MAXIMIZED)
 			geo[4] = 1;
 		else
 			geo[4] = 0;
@@ -174,7 +140,26 @@ static gboolean gigolo_window_delete_event(GtkWidget *widget, G_GNUC_UNUSED GdkE
 	g_object_unref(priv->systray_icon_popup_menu);
 	g_object_unref(priv->backend_gvfs);
 
-	return FALSE;
+	gtk_widget_destroy(GTK_WIDGET(window));
+
+	gtk_main_quit();
+}
+
+
+static gboolean gigolo_window_delete_event(GtkWidget *widget, G_GNUC_UNUSED GdkEventAny *event)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(widget);
+
+	if (gigolo_settings_get_boolean(priv->settings, "show-in-systray"))
+	{
+		gtk_widget_hide(widget);
+		return TRUE;
+	}
+	else
+	{
+		gigolo_window_destroy(GIGOLO_WINDOW(widget));
+		return FALSE;
+	}
 }
 
 
@@ -182,7 +167,6 @@ static void gigolo_window_class_init(GigoloWindowClass *klass)
 {
 	GtkWidgetClass *gtkwidget_class = GTK_WIDGET_CLASS(klass);
 	gtkwidget_class->delete_event = gigolo_window_delete_event;
-	gtkwidget_class->window_state_event = gigolo_window_state_event;
 
 	g_type_class_add_private(klass, sizeof(GigoloWindowPrivate));
 }
@@ -243,15 +227,21 @@ static void get_selected_iter(GigoloWindow *window, GtkTreeIter *iter)
 }
 
 
-static GigoloBookmark *get_bookmark_from_uri(GigoloWindow *window, const gchar *uri)
+GigoloBookmark *gigolo_window_find_bookmark_by_uri(GigoloWindow *window, const gchar *uri)
 {
-	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
-	GigoloBookmarkList *bml = gigolo_settings_get_bookmarks(priv->settings);
+	GigoloWindowPrivate *priv;
+	GigoloBookmarkList *bml;
 	GigoloBookmark *bm = NULL;
 	gboolean found = FALSE;
 	gchar *tmp_uri;
 	guint i;
 
+	g_return_val_if_fail(window != NULL, FALSE);
+	g_return_val_if_fail(uri != NULL, FALSE);
+
+	priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	bml = gigolo_settings_get_bookmarks(priv->settings);
+
 	for (i = 0; i < bml->len && ! found; i++)
 	{
 		bm = g_ptr_array_index(bml, i);
@@ -265,7 +255,8 @@ static GigoloBookmark *get_bookmark_from_uri(GigoloWindow *window, const gchar *
 }
 
 
-static void mount_from_bookmark(GigoloWindow *window, GigoloBookmark *bookmark, gboolean show_dialog)
+void gigolo_window_mount_from_bookmark(GigoloWindow *window, GigoloBookmark *bookmark,
+									   gboolean show_dialog, gboolean show_errors)
 {
 	gchar *uri;
 	GtkWidget *dialog = NULL;
@@ -289,7 +280,7 @@ static void mount_from_bookmark(GigoloWindow *window, GigoloBookmark *bookmark,
 		g_free(label);
 	}
 
-	gigolo_backend_gvfs_mount_uri(priv->backend_gvfs, uri, gigolo_bookmark_get_domain(bookmark), dialog);
+	gigolo_backend_gvfs_mount_uri(priv->backend_gvfs, uri, GTK_WINDOW(window), dialog, show_errors);
 
 	if (gigolo_bookmark_get_autoconnect(bookmark))
 		gigolo_bookmark_set_should_not_autoconnect(bookmark, FALSE);
@@ -323,15 +314,14 @@ static void action_mount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *windo
 		GigoloBookmark *bm = NULL;
 		GtkWidget *dialog;
 
-		dialog = gigolo_bookmark_edit_dialog_new(GTK_WINDOW(window),
-			priv->settings, GIGOLO_BE_MODE_CONNECT);
+		dialog = gigolo_bookmark_edit_dialog_new(window, GIGOLO_BE_MODE_CONNECT);
 		if (gigolo_bookmark_edit_dialog_run(GIGOLO_BOOKMARK_EDIT_DIALOG(dialog)) == GTK_RESPONSE_OK)
 		{
 			bm = gigolo_bookmark_new();
 			/* this fills the values of the dialog into 'bm' */
 			g_object_set(dialog, "bookmark-update", bm, NULL);
 
-			mount_from_bookmark(window, bm, TRUE);
+			gigolo_window_mount_from_bookmark(window, bm, TRUE, TRUE);
 
 			g_object_unref(bm);
 		}
@@ -355,6 +345,34 @@ static void action_preferences_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow
 }
 
 
+static void action_toggle_view_cb(GtkToggleAction *action, GigoloWindow *window)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	gboolean active = gtk_toggle_action_get_active(action);
+	const gchar *property = NULL;
+
+	if (gtk_action_get_name(GTK_ACTION(action)) == g_intern_string("ShowPanel"))
+		property = "show-panel";
+	else if (gtk_action_get_name(GTK_ACTION(action)) == g_intern_string("ShowToolbar"))
+		property = "show-toolbar";
+	else if (gtk_action_get_name(GTK_ACTION(action)) == g_intern_string("ShowInSystray"))
+		property = "show-in-systray";
+
+	if (property != NULL)
+		g_object_set(priv->settings, property, active, NULL);
+}
+
+
+static void action_view_mode_change_cb(G_GNUC_UNUSED GtkRadioAction *action,
+									   GtkRadioAction *current, GigoloWindow *window)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	gint mode = gtk_radio_action_get_current_value(current);
+
+	g_object_set(priv->settings, "view-mode", mode, NULL);
+}
+
+
 static void action_unmount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
@@ -372,7 +390,7 @@ static void action_unmount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *win
 		{
 			gchar *uri;
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, NULL, &uri);
-			bm = get_bookmark_from_uri(window, uri);
+			bm = gigolo_window_find_bookmark_by_uri(window, uri);
 			if (bm != NULL && gigolo_bookmark_get_autoconnect(bm))
 			{	/* we don't want auto-connection to reconnect this bookmark right
 				   after we unmount it. */
@@ -388,8 +406,7 @@ static void action_unmount_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *win
 
 static void action_quit_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window)
 {
-    gigolo_window_delete_event(GTK_WIDGET(window), NULL);
-    gtk_widget_destroy(GTK_WIDGET(window));
+    gigolo_window_destroy(window);
 }
 
 
@@ -398,7 +415,7 @@ static void action_bookmark_edit_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindo
 	GtkWidget *dialog;
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 
-	dialog = gigolo_bookmark_dialog_new(GTK_WINDOW(window), priv->settings);
+	dialog = gigolo_bookmark_dialog_new(window);
 
 	gtk_dialog_run(GTK_DIALOG(dialog));
 	gigolo_settings_write(priv->settings, GIGOLO_SETTINGS_BOOKMARKS);
@@ -445,6 +462,45 @@ static void action_about_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *windo
 }
 
 
+static void action_help_cb(G_GNUC_UNUSED GtkAction *action, G_GNUC_UNUSED GigoloWindow *window)
+{
+	gigolo_show_uri("http://www.uvena.de/gigolo/help.html");
+}
+
+
+static void action_supported_schemes_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window)
+{
+	const gchar* const *supported;
+	const gchar *description;
+	GString *str;
+	GtkWidget *dialog;
+	guint j;
+
+	str = g_string_new(_("Gigolo can use the following protocols provided by GVfs:"));
+	g_string_append(str, "\n\n");
+
+	supported = gigolo_backend_gvfs_get_supported_uri_schemes();
+	for (j = 0; supported[j] != NULL; j++)
+	{
+		description = gigolo_describe_scheme(supported[j]);
+		if (description != NULL)
+		{
+			g_string_append_printf(str, "%s (%s)", description, supported[j]);
+			g_string_append_c(str, '\n');
+		}
+	}
+
+	dialog = gtk_message_dialog_new(GTK_WINDOW(window),
+		GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_INFO,
+		GTK_BUTTONS_OK, "%s", str->str);
+
+	gtk_dialog_run(GTK_DIALOG(dialog));
+
+	gtk_widget_destroy(dialog);
+	g_string_free(str, TRUE);
+}
+
+
 static void action_copy_uri_cb(G_GNUC_UNUSED GtkAction *action, GigoloWindow *window)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
@@ -558,7 +614,7 @@ static gboolean iter_is_bookmark(GigoloWindow *window, GtkTreeModel *model, GtkT
 
 		gigolo_backend_gvfs_get_name_and_uri_from_mount(ref, NULL, &uri);
 
-		found = (get_bookmark_from_uri(window, uri) != NULL);
+		found = (gigolo_window_find_bookmark_by_uri(window, uri) != NULL);
 
 		g_free(uri);
 		return found;
@@ -746,7 +802,7 @@ static void action_bookmark_activate_cb(G_GNUC_UNUSED GigoloMenubuttonAction *ac
 {
 	GigoloBookmark *bm = g_object_get_data(G_OBJECT(item), "bookmark");
 
-	mount_from_bookmark(window, bm, TRUE);
+	gigolo_window_mount_from_bookmark(window, bm, TRUE, TRUE);
 }
 
 
@@ -782,6 +838,7 @@ gboolean gigolo_window_do_autoconnect(gpointer data)
 	static gint old_interval = -1;
 	gint interval;
 	guint i;
+	gboolean show_errors;
 
 	interval = gigolo_settings_get_integer(priv->settings, "autoconnect-interval");
 	if (old_interval != interval)
@@ -800,12 +857,13 @@ gboolean gigolo_window_do_autoconnect(gpointer data)
 		return FALSE;
 	}
 
+	show_errors = gigolo_settings_get_boolean(priv->settings, "show-autoconnect-errors");
 	for (i = 0; i < bookmarks->len; i++)
 	{
 		GigoloBookmark *bm = g_ptr_array_index(bookmarks, i);
 		if (gigolo_bookmark_get_autoconnect(bm) && ! gigolo_bookmark_get_should_not_autoconnect(bm))
 		{
-			mount_from_bookmark(window, bm, FALSE);
+			gigolo_window_mount_from_bookmark(window, bm, FALSE, show_errors);
 		}
 	}
 	return TRUE;
@@ -830,7 +888,7 @@ static void action_create_bookmark_cb(G_GNUC_UNUSED GtkAction *button, GigoloWin
 
 			gigolo_backend_gvfs_get_name_and_uri_from_mount(mnt, &name, &uri);
 
-			if (get_bookmark_from_uri(window, uri) == NULL)
+			if (gigolo_window_find_bookmark_by_uri(window, uri) == NULL)
 			{
 				GigoloBookmark *bm = gigolo_bookmark_new_from_uri(name, uri);
 				if (gigolo_bookmark_is_valid(bm))
@@ -840,9 +898,9 @@ static void action_create_bookmark_cb(G_GNUC_UNUSED GtkAction *button, GigoloWin
 					/* show the bookmark edit dialog and add the bookmark only if it was
 					 * not cancelled */
 					edit_dialog = gigolo_bookmark_edit_dialog_new_with_bookmark(
-						GTK_WINDOW(window), priv->settings, GIGOLO_BE_MODE_EDIT, bm);
-					if (gigolo_bookmark_edit_dialog_run(GIGOLO_BOOKMARK_EDIT_DIALOG(edit_dialog)) ==
-						GTK_RESPONSE_OK)
+						window, GIGOLO_BE_MODE_EDIT, bm);
+					if (gigolo_bookmark_edit_dialog_run(
+							GIGOLO_BOOKMARK_EDIT_DIALOG(edit_dialog)) == GTK_RESPONSE_OK)
 					{
 						/* this fills the values of the dialog into 'bm' */
 						g_object_set(edit_dialog, "bookmark-update", bm, NULL);
@@ -866,6 +924,17 @@ static void action_create_bookmark_cb(G_GNUC_UNUSED GtkAction *button, GigoloWin
 }
 
 
+static void gigolo_window_show_side_panel(GigoloWindow *window, gboolean show)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+
+	if (show && gigolo_backend_gvfs_is_scheme_supported("smb"))
+		gtk_widget_show(priv->hbox_pane);
+	else
+		gtk_widget_hide(priv->hbox_pane);
+}
+
+
 static void gigolo_window_show_systray_icon(GigoloWindow *window, gboolean show)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
@@ -907,16 +976,16 @@ static void gigolo_window_set_toolbar_orientation(GigoloWindow *window, gint ori
 	gigolo_toolbar_set_orientation(GTK_TOOLBAR(priv->toolbar), orientation);
 	if (orientation == GTK_ORIENTATION_HORIZONTAL && priv->vbox != gtk_widget_get_parent(priv->toolbar))
 	{
-		gtk_container_remove(GTK_CONTAINER(priv->hbox), priv->toolbar);
+		gtk_container_remove(GTK_CONTAINER(priv->hbox_view), priv->toolbar);
 		gtk_container_add(GTK_CONTAINER(priv->vbox), priv->toolbar);
 		gtk_box_set_child_packing(GTK_BOX(priv->vbox), priv->toolbar, FALSE, FALSE, 0, GTK_PACK_START);
 		gtk_box_reorder_child(GTK_BOX(priv->vbox), priv->toolbar, 1);
 	}
-	else if (orientation == GTK_ORIENTATION_VERTICAL && priv->hbox != gtk_widget_get_parent(priv->toolbar))
+	else if (orientation == GTK_ORIENTATION_VERTICAL && priv->hbox_view != gtk_widget_get_parent(priv->toolbar))
 	{
 		gtk_container_remove(GTK_CONTAINER(priv->vbox), priv->toolbar);
-		gtk_container_add(GTK_CONTAINER(priv->hbox), priv->toolbar);
-		gtk_box_set_child_packing(GTK_BOX(priv->hbox), priv->toolbar, FALSE, FALSE, 0, GTK_PACK_START);
+		gtk_container_add(GTK_CONTAINER(priv->hbox_view), priv->toolbar);
+		gtk_box_set_child_packing(GTK_BOX(priv->hbox_view), priv->toolbar, FALSE, FALSE, 0, GTK_PACK_START);
 	}
 }
 
@@ -925,21 +994,48 @@ static void gigolo_window_set_view_mode(GigoloWindow *window, gint mode)
 {
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 
-	if (mode == VIEW_MODE_ICONVIEW && priv->hbox != gtk_widget_get_parent(priv->swin_iconview))
+	if (mode == VIEW_MODE_ICONVIEW && priv->hbox_view != gtk_widget_get_parent(priv->swin_iconview))
 	{
 		gtk_tree_selection_unselect_all(gtk_tree_view_get_selection(GTK_TREE_VIEW(priv->treeview)));
-		gtk_container_remove(GTK_CONTAINER(priv->hbox), priv->swin_treeview);
-		gtk_container_add(GTK_CONTAINER(priv->hbox), priv->swin_iconview);
+		gtk_container_remove(GTK_CONTAINER(priv->hbox_view), priv->swin_treeview);
+		gtk_container_add(GTK_CONTAINER(priv->hbox_view), priv->swin_iconview);
 	}
-	else if (mode == VIEW_MODE_TREEVIEW &&  priv->hbox != gtk_widget_get_parent(priv->swin_treeview))
+	else if (mode == VIEW_MODE_TREEVIEW &&  priv->hbox_view != gtk_widget_get_parent(priv->swin_treeview))
 	{
 		gtk_icon_view_unselect_all(GTK_ICON_VIEW(priv->iconview));
-		gtk_container_remove(GTK_CONTAINER(priv->hbox), priv->swin_iconview);
-		gtk_container_add(GTK_CONTAINER(priv->hbox), priv->swin_treeview);
+		gtk_container_remove(GTK_CONTAINER(priv->hbox_view), priv->swin_iconview);
+		gtk_container_add(GTK_CONTAINER(priv->hbox_view), priv->swin_treeview);
 	}
 }
 
 
+static void toggle_action_set_active(GigoloWindow *window, const gchar *name, gboolean set)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	GtkAction *action = gtk_action_group_get_action(priv->action_group, name);
+
+	gtk_toggle_action_set_active(GTK_TOGGLE_ACTION(action), set);
+}
+
+
+static void view_mode_action_set_active(GigoloWindow *window, gint val)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	GtkAction *action = gtk_action_group_get_action(priv->action_group, "ViewSymbols");
+
+	gtk_radio_action_set_current_value(GTK_RADIO_ACTION(action), val);
+}
+
+
+static void action_set_sensitive(GigoloWindow *window, const gchar *name, gboolean set)
+{
+	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+	GtkAction *action = gtk_action_group_get_action(priv->action_group, name);
+
+	gtk_action_set_sensitive(action, set);
+}
+
+
 static void gigolo_window_settings_notify_cb(GigoloSettings *settings, GParamSpec *pspec, GigoloWindow *window)
 {
 	const gchar *name;
@@ -951,15 +1047,33 @@ static void gigolo_window_settings_notify_cb(GigoloSettings *settings, GParamSpe
 	g_object_get_property(G_OBJECT(settings), name, value);
 
 	if (name == g_intern_string("show-toolbar"))
-		gigolo_window_show_toolbar(window, g_value_get_boolean(value));
+	{
+		gboolean state = g_value_get_boolean(value);
+		gigolo_window_show_toolbar(window, state);
+		toggle_action_set_active(window, "ShowToolbar", state);
+	}
 	else if (name == g_intern_string("show-in-systray"))
-		gigolo_window_show_systray_icon(window, g_value_get_boolean(value));
+	{
+		gboolean state = g_value_get_boolean(value);
+		gigolo_window_show_systray_icon(window, state);
+		toggle_action_set_active(window, "ShowInSystray", state);
+	}
 	else if (name == g_intern_string("toolbar-style"))
 		gigolo_window_set_toolbar_style(window, g_value_get_int(value));
 	else if (name == g_intern_string("toolbar-orientation"))
 		gigolo_window_set_toolbar_orientation(window, g_value_get_int(value));
 	else if (name == g_intern_string("view-mode"))
-		gigolo_window_set_view_mode(window, g_value_get_int(value));
+	{
+		gint mode = g_value_get_int(value);
+		gigolo_window_set_view_mode(window, mode);
+		view_mode_action_set_active(GIGOLO_WINDOW(window), mode);
+	}
+	else if (name == g_intern_string("show-panel"))
+	{
+		gboolean state = g_value_get_boolean(value);
+		gigolo_window_show_side_panel(window, state);
+		toggle_action_set_active(window, "ShowPanel", state);
+	}
 	else if (! g_object_class_find_property(G_OBJECT_GET_CLASS(settings), name))
 		 verbose("Unexpected setting '%s'", name);
 	g_value_unset(value);
@@ -982,6 +1096,14 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 				"<separator/>"
 				"<menuitem action='Preferences'/>"
 			"</menu>"
+			"<menu action='View'>"
+				"<menuitem action='ShowToolbar'/>"
+				"<menuitem action='ShowPanel'/>"
+				"<menuitem action='ShowInSystray'/>"
+				"<separator/>"
+				"<menuitem action='ViewDetailed'/>"
+				"<menuitem action='ViewSymbols'/>"
+			"</menu>"
 			"<menu action='Actions'>"
 				"<menuitem action='Connect'/>"
 				"<menuitem action='Disconnect'/>"
@@ -991,6 +1113,9 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 				"<menuitem action='CopyURI'/>"
 			"</menu>"
 			"<menu action='Help'>"
+				"<menuitem action='OnlineHelp'/>"
+				"<menuitem action='SupportedSchemes'/>"
+				"<separator/>"
 				"<menuitem action='About'/>"
 			"</menu>"
 		"</menubar>"
@@ -1029,6 +1154,7 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 		{ "File", NULL, N_("_File"), NULL, NULL, NULL },
 		{ "Edit", NULL, N_("_Edit"), NULL, NULL, NULL },
 		{ "Actions", NULL, N_("_Actions"), NULL, NULL, NULL },
+		{ "View", NULL, N_("_View"), NULL, NULL, NULL },
 		{ "Help", NULL, N_("_Help"), NULL, NULL, NULL },
 		{ "Preferences", GTK_STOCK_PREFERENCES,
 			NULL, "<Ctrl>p", NULL, G_CALLBACK(action_preferences_cb) },
@@ -1045,10 +1171,28 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 			N_("Open the selected resource with a file manager"), G_CALLBACK(action_open_cb) },
 		{ "CopyURI", GTK_STOCK_COPY, N_("Copy _URI"), "<Ctrl>c", NULL, G_CALLBACK(action_copy_uri_cb) },
 		{ "Quit", GTK_STOCK_QUIT, NULL, "<Ctrl>q", N_("Quit Gigolo"), G_CALLBACK(action_quit_cb) },
+		{ "OnlineHelp", GTK_STOCK_HELP, _("Online Help"), NULL, NULL, G_CALLBACK(action_help_cb) },
+		{ "SupportedSchemes", NULL, _("Supported Protocols"), NULL, NULL, G_CALLBACK(action_supported_schemes_cb) },
 		{ "About", GTK_STOCK_ABOUT, NULL, NULL, NULL, G_CALLBACK(action_about_cb) }
 	};
 	const guint entries_n = G_N_ELEMENTS(entries);
 
+	const GtkToggleActionEntry toggle_entries[] = {
+		{ "ShowPanel", NULL, N_("Side _Panel"),
+			NULL, NULL, G_CALLBACK(action_toggle_view_cb), FALSE },
+		{ "ShowToolbar", NULL, N_("_Toolbar"),
+			NULL, NULL, G_CALLBACK(action_toggle_view_cb), FALSE },
+		{ "ShowInSystray", NULL, N_("Status _Icon"),
+			NULL, NULL, G_CALLBACK(action_toggle_view_cb), FALSE }
+	};
+	const guint toggle_entries_n = G_N_ELEMENTS(toggle_entries);
+
+	const GtkRadioActionEntry radio_entries[] = {
+		{ "ViewDetailed", NULL, N_("View as _Symbols"), NULL, NULL, 0 },
+		{ "ViewSymbols", NULL, N_("View as _Detailed List"), NULL, NULL, 1 },
+	};
+	const guint radio_entries_n = G_N_ELEMENTS(radio_entries);
+
 
 	priv->action_bookmarks = gigolo_menu_button_action_new(
 		"Bookmarks", _("_Bookmarks"), _("Choose a bookmark to connect to"),
@@ -1060,6 +1204,9 @@ static void create_ui_elements(GigoloWindow *window, GtkUIManager *ui_manager)
 	priv->action_group = gtk_action_group_new("UI");
 	gtk_action_group_set_translation_domain(priv->action_group, GETTEXT_PACKAGE);
 	gtk_action_group_add_actions(priv->action_group, entries, entries_n, window);
+	gtk_action_group_add_toggle_actions(priv->action_group, toggle_entries, toggle_entries_n, window);
+	gtk_action_group_add_radio_actions(priv->action_group, radio_entries, radio_entries_n, -1,
+		G_CALLBACK(action_view_mode_change_cb), window);
 	gtk_action_group_add_action(priv->action_group, priv->action_bookmarks);
 	gtk_ui_manager_insert_action_group(ui_manager, priv->action_group, 0);
 	gtk_window_add_accel_group(GTK_WINDOW(window), gtk_ui_manager_get_accel_group(ui_manager));
@@ -1200,7 +1347,7 @@ static void create_icon_view(GigoloWindow *window)
 
 static void gigolo_window_init(GigoloWindow *window)
 {
-	GtkWidget *menubar;
+	GtkWidget *menubar, *panel_pane;
 	GtkUIManager *ui_manager;
 	GigoloWindowPrivate *priv = GIGOLO_WINDOW_GET_PRIVATE(window);
 
@@ -1208,7 +1355,7 @@ static void gigolo_window_init(GigoloWindow *window)
 
 	gtk_window_set_title(GTK_WINDOW(window), _("Gigolo"));
 	gtk_window_set_icon_name(GTK_WINDOW(window), gigolo_get_application_icon_name());
-	gtk_window_set_default_size(GTK_WINDOW(window), 550, 350);
+	gtk_window_set_default_size(GTK_WINDOW(window), 650, 350);
 
 	/* Init liststore */
 	priv->store = gtk_list_store_new(GIGOLO_WINDOW_N_COLUMNS,
@@ -1257,15 +1404,27 @@ static void gigolo_window_init(GigoloWindow *window)
 	priv->action_open = gtk_action_group_get_action(priv->action_group, "Open");
 	priv->action_copyuri = gtk_action_group_get_action(priv->action_group, "CopyURI");
 
+	/* Panel */
+	panel_pane = gtk_hpaned_new();
+	gtk_paned_set_position(GTK_PANED(panel_pane), 200);
+
+	priv->browse_panel = gigolo_browse_network_panel_new(window);
+	gtk_widget_show(priv->browse_panel);
+
 	/* Pack the widgets altogether */
 	priv->vbox = gtk_vbox_new(FALSE, 0);
-	priv->hbox = gtk_hbox_new(FALSE, 0);
+	priv->hbox_view = gtk_hbox_new(FALSE, 0);
+	priv->hbox_pane = gtk_hbox_new(FALSE, 0);
+
+	gtk_paned_add1(GTK_PANED(panel_pane), priv->hbox_pane);
+	gtk_paned_add2(GTK_PANED(panel_pane), priv->hbox_view);
+	gtk_box_pack_start(GTK_BOX(priv->hbox_pane), priv->browse_panel, TRUE, TRUE, 0);
 
 	gtk_box_pack_start(GTK_BOX(priv->vbox), menubar, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX(priv->vbox), priv->toolbar, FALSE, FALSE, 0);
 
-	gtk_box_pack_start(GTK_BOX(priv->hbox), priv->swin_iconview, TRUE, TRUE, 0);
-	gtk_box_pack_start(GTK_BOX(priv->vbox), priv->hbox, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(priv->hbox_view), priv->swin_iconview, TRUE, TRUE, 0);
+	gtk_box_pack_start(GTK_BOX(priv->vbox), panel_pane, TRUE, TRUE, 0);
 
 	gtk_container_add(GTK_CONTAINER(window), priv->vbox);
 
@@ -1288,6 +1447,8 @@ GtkWidget *gigolo_window_new(GigoloSettings *settings)
 	GtkWidget *window;
 	GigoloWindowPrivate *priv;
 	const gint *geo;
+	gboolean state;
+	gint value;
 
 	window = g_object_new(GIGOLO_WINDOW_TYPE, NULL);
 	priv = GIGOLO_WINDOW_GET_PRIVATE(window);
@@ -1296,16 +1457,27 @@ GtkWidget *gigolo_window_new(GigoloSettings *settings)
 
 	g_object_set(priv->action_bookmarks, "settings", settings, NULL);
 
-	gigolo_window_show_systray_icon(GIGOLO_WINDOW(window),
-		gigolo_settings_get_boolean(settings, "show-in-systray"));
-	gigolo_window_show_toolbar(GIGOLO_WINDOW(window),
-		gigolo_settings_get_boolean(settings, "show-toolbar"));
 	gigolo_window_set_toolbar_style(GIGOLO_WINDOW(window),
 		gigolo_settings_get_integer(settings, "toolbar-style"));
 	gigolo_window_set_toolbar_orientation(GIGOLO_WINDOW(window),
 		gigolo_settings_get_integer(settings, "toolbar-orientation"));
-	gigolo_window_set_view_mode(GIGOLO_WINDOW(window),
-		gigolo_settings_get_integer(settings, "view-mode"));
+	/* Show Panel */
+	state = gigolo_settings_get_boolean(settings, "show-panel");
+	gigolo_window_show_side_panel(GIGOLO_WINDOW(window), state);
+	toggle_action_set_active(GIGOLO_WINDOW(window), "ShowPanel", state);
+	action_set_sensitive(GIGOLO_WINDOW(window), "ShowPanel", gigolo_backend_gvfs_is_scheme_supported("smb"));
+	/* Show Toolbar */
+	state = gigolo_settings_get_boolean(settings, "show-toolbar");
+	gigolo_window_show_toolbar(GIGOLO_WINDOW(window), state);
+	toggle_action_set_active(GIGOLO_WINDOW(window), "ShowToolbar", state);
+	/* Show Status Icon */
+	state = gigolo_settings_get_boolean(settings, "show-in-systray");
+	gigolo_window_show_systray_icon(GIGOLO_WINDOW(window), state);
+	toggle_action_set_active(GIGOLO_WINDOW(window), "ShowInSystray", state);
+	/* View Mode */
+	value = gigolo_settings_get_integer(settings, "view-mode");
+	gigolo_window_set_view_mode(GIGOLO_WINDOW(window), value);
+	view_mode_action_set_active(GIGOLO_WINDOW(window), value);
 
 	if (gigolo_settings_get_boolean(settings, "save-geometry"))
 	{
@@ -1327,3 +1499,15 @@ GtkWidget *gigolo_window_new(GigoloSettings *settings)
 }
 
 
+GigoloSettings *gigolo_window_get_settings(GigoloWindow *window)
+{
+	GigoloWindowPrivate *priv;
+
+	g_return_val_if_fail(window != NULL, NULL);
+
+	priv = GIGOLO_WINDOW_GET_PRIVATE(window);
+
+	return priv->settings;
+}
+
+
